/* lpcboot - an LPC17xx MMC/SD boot loader inspired by HolgerBootloader2
   Based on a heavily modified version of ChaN's FatFs library

   Copyright (C) 2013  Ingo Korb <ingo@akana.de>
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.


   lpc_iap.h: Definitions for the interface to the IAP ROM

*/

#ifndef LPC_IAP_H
#define LPC_IAP_H

typedef enum { IR_SUCCESS,
               IR_INVALID_CMD,
               IR_SRC_ADDR_ERROR,
               IR_DST_ADDR_ERROR,
               IR_SRC_ADDR_NOT_MAPPED,
               IR_DST_ADDR_NOT_MAPPED,
               IR_COUNT_ERROR,
               IR_INVALID_SECTOR,
               IR_SECTOR_NOT_BLANK,
               IR_SECTOR_NOT_PREPARED,
               IR_COMPARE_ERROR,
               IR_BUSY } iapresult_t;

#define LPC_SECTOR_COUNT 30

extern unsigned char sectorsizes[LPC_SECTOR_COUNT+1];

/* WARNING: All functions returning iapresult_t must be called with interrupts disabled! */

unsigned int iap_address_to_sector(void *address);
iapresult_t  iap_read_partid(unsigned long *id);
iapresult_t  iap_read_serial(unsigned long serial[const 4]);
iapresult_t  iap_read_bootcode_version(unsigned long *version);
iapresult_t  iap_blank_check(unsigned int startsector, unsigned int endsector);
iapresult_t  iap_prepare(unsigned int startsector, unsigned int endsector);
iapresult_t  iap_erase(unsigned int startsector, unsigned int endsector, unsigned int cpu_khz);
iapresult_t  iap_ram2flash(void *dest, void *src, unsigned int bytes, unsigned int cpu_khz);
iapresult_t  iap_compare(void *addr1, void *addr2, unsigned int bytes);
void         iap_start_isp(void);

#endif
