/* lpcboot - an LPC17xx MMC/SD boot loader inspired by HolgerBootloader2
   Based on a heavily modified version of ChaN's FatFs library

   Copyright (C) 2013  Ingo Korb <ingo@akana.de>
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.


   system.c: system clock initialisation

*/

#include "config.h"
#include <arm/NXP/LPC17xx/LPC17xx.h>
#include <arm/bits.h>
#include "system.h"

void system_speedup(void) {
  /* Set flash accelerator to 5 CPU cycle access time */
  LPC_SC->FLASHCFG = (LPC_SC->FLASHCFG & 0xffff0fff) | (4 << 12);

  /* Enable main oscillator, range 1-20MHz */
  BITBAND(LPC_SC->SCS, 5) = 1;

  /* Wait until stable */
  while (!BITBAND(LPC_SC->SCS, 6)) ;

  /* Use main oscillator as system clock source */
  LPC_SC->CLKSRCSEL = 1;

  /* Set up PLL0 multiplier and pre-divisor */
  LPC_SC->PLL0CFG  = ((PLL_PREDIV-1) << 16) | (PLL_MULTIPLIER-1);
  LPC_SC->PLL0FEED = 0xaa;
  LPC_SC->PLL0FEED = 0x55;

  /* Enable PLL0 */
  LPC_SC->PLL0CON  = 1;
  LPC_SC->PLL0FEED = 0xaa;
  LPC_SC->PLL0FEED = 0x55;

  /* Increase CPU clock divider */
  LPC_SC->CCLKCFG = PLL_DIVISOR-1;

  /* Wait until PLL locks */
  while (!(LPC_SC->PLL0STAT & BV(26))) ;

  /* Connect PLL0 */
  LPC_SC->PLL0CON  = 3;
  LPC_SC->PLL0FEED = 0xaa;
  LPC_SC->PLL0FEED = 0x55;
}

void system_deinit(void) {
  /* disconnect PLL0 */
  LPC_SC->PLL0CON = 1;
  LPC_SC->PLL0FEED = 0xaa;
  LPC_SC->PLL0FEED = 0x55;

  /* reset CPU clock divider */
  LPC_SC->CCLKCFG = 0;

  /* disable PLL0 */
  LPC_SC->PLL0CON  = 0;
  LPC_SC->PLL0FEED = 0xaa;
  LPC_SC->PLL0FEED = 0x55;

  /* reset PLL0 settings */
  LPC_SC->PLL0CFG = 0;
  LPC_SC->PLL0FEED = 0xaa;
  LPC_SC->PLL0FEED = 0x55;

  /* use IRC as system clock source */
  LPC_SC->CLKSRCSEL = 0;

  /* disable main oscillator */
  LPC_SC->SCS = 0;

  /* reset flash accelerator setting */
  LPC_SC->FLASHCFG = (LPC_SC->FLASHCFG & 0xffff0fff) | (3 << 12);
}
